/****************************************************************************
** Meta object code from reading C++ file 'fetcher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../telly-skout-26.08.0/src/fetcher.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fetcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7FetcherE_t {};
} // unnamed namespace

template <> constexpr inline auto Fetcher::qt_create_metaobjectdata<qt_meta_tag_ZN7FetcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Fetcher",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "favoritesPercentageChanged",
        "",
        "percentage",
        "startedFetchingGroup",
        "GroupId",
        "id",
        "groupUpdated",
        "startedFetchingChannel",
        "ChannelId",
        "finishedFetchingChannel",
        "channelUpdated",
        "programDescriptionUpdated",
        "ProgramId",
        "description",
        "errorFetchingGroup",
        "Error",
        "error",
        "errorFetchingChannel",
        "imageDownloadFinished",
        "url",
        "errorDownloadingImage",
        "fetchFavorites",
        "force",
        "fetchGroups",
        "fetchGroup",
        "groupId",
        "fetchProgramDescription",
        "channelId",
        "programId",
        "image",
        "favoritesPercentage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'favoritesPercentageChanged'
        QtMocHelpers::SignalData<void(unsigned int)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 7 },
        }}),
        // Signal 'startedFetchingGroup'
        QtMocHelpers::SignalData<void(const GroupId &)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'groupUpdated'
        QtMocHelpers::SignalData<void(const GroupId &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'startedFetchingChannel'
        QtMocHelpers::SignalData<void(const ChannelId &)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 10 },
        }}),
        // Signal 'finishedFetchingChannel'
        QtMocHelpers::SignalData<void(const ChannelId &)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 10 },
        }}),
        // Signal 'channelUpdated'
        QtMocHelpers::SignalData<void(const ChannelId &)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 10 },
        }}),
        // Signal 'programDescriptionUpdated'
        QtMocHelpers::SignalData<void(const ProgramId &, const QString &)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 10 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'errorFetchingGroup'
        QtMocHelpers::SignalData<void(const GroupId &, const Error &)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'errorFetchingChannel'
        QtMocHelpers::SignalData<void(const ChannelId &, const Error &)>(22, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 10 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'imageDownloadFinished'
        QtMocHelpers::SignalData<void(const QString &)>(23, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'errorDownloadingImage'
        QtMocHelpers::SignalData<void(const QString &, const Error &)>(25, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 }, { 0x80000000 | 20, 21 },
        }}),
        // Method 'fetchFavorites'
        QtMocHelpers::MethodData<void(bool)>(26, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Method 'fetchFavorites'
        QtMocHelpers::MethodData<void()>(26, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'fetchGroups'
        QtMocHelpers::MethodData<void()>(28, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'fetchGroup'
        QtMocHelpers::MethodData<void(const QString &, const GroupId &)>(29, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 }, { 0x80000000 | 9, 30 },
        }}),
        // Method 'fetchProgramDescription'
        QtMocHelpers::MethodData<void(const ChannelId &, const ProgramId &, const QString &)>(31, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 32 }, { 0x80000000 | 17, 33 }, { QMetaType::QString, 24 },
        }}),
        // Method 'image'
        QtMocHelpers::MethodData<QString(const QString &)>(34, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'favoritesPercentage'
        QtMocHelpers::PropertyData<uint>(35, QMetaType::UInt, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<Fetcher, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Fetcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7FetcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7FetcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7FetcherE_t>.metaTypes,
    nullptr
} };

void Fetcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Fetcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->favoritesPercentageChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 1: _t->startedFetchingGroup((*reinterpret_cast<std::add_pointer_t<GroupId>>(_a[1]))); break;
        case 2: _t->groupUpdated((*reinterpret_cast<std::add_pointer_t<GroupId>>(_a[1]))); break;
        case 3: _t->startedFetchingChannel((*reinterpret_cast<std::add_pointer_t<ChannelId>>(_a[1]))); break;
        case 4: _t->finishedFetchingChannel((*reinterpret_cast<std::add_pointer_t<ChannelId>>(_a[1]))); break;
        case 5: _t->channelUpdated((*reinterpret_cast<std::add_pointer_t<ChannelId>>(_a[1]))); break;
        case 6: _t->programDescriptionUpdated((*reinterpret_cast<std::add_pointer_t<ProgramId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->errorFetchingGroup((*reinterpret_cast<std::add_pointer_t<GroupId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Error>>(_a[2]))); break;
        case 8: _t->errorFetchingChannel((*reinterpret_cast<std::add_pointer_t<ChannelId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Error>>(_a[2]))); break;
        case 9: _t->imageDownloadFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->errorDownloadingImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Error>>(_a[2]))); break;
        case 11: _t->fetchFavorites((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->fetchFavorites(); break;
        case 13: _t->fetchGroups(); break;
        case 14: _t->fetchGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GroupId>>(_a[2]))); break;
        case 15: _t->fetchProgramDescription((*reinterpret_cast<std::add_pointer_t<ChannelId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ProgramId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 16: { QString _r = _t->image((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(unsigned int )>(_a, &Fetcher::favoritesPercentageChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const GroupId & )>(_a, &Fetcher::startedFetchingGroup, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const GroupId & )>(_a, &Fetcher::groupUpdated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const ChannelId & )>(_a, &Fetcher::startedFetchingChannel, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const ChannelId & )>(_a, &Fetcher::finishedFetchingChannel, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const ChannelId & )>(_a, &Fetcher::channelUpdated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const ProgramId & , const QString & )>(_a, &Fetcher::programDescriptionUpdated, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const GroupId & , const Error & )>(_a, &Fetcher::errorFetchingGroup, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const ChannelId & , const Error & )>(_a, &Fetcher::errorFetchingChannel, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const QString & )>(_a, &Fetcher::imageDownloadFinished, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fetcher::*)(const QString & , const Error & )>(_a, &Fetcher::errorDownloadingImage, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->favoritesPercentage(); break;
        default: break;
        }
    }
}

const QMetaObject *Fetcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Fetcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7FetcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Fetcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Fetcher::favoritesPercentageChanged(unsigned int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Fetcher::startedFetchingGroup(const GroupId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Fetcher::groupUpdated(const GroupId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Fetcher::startedFetchingChannel(const ChannelId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Fetcher::finishedFetchingChannel(const ChannelId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Fetcher::channelUpdated(const ChannelId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Fetcher::programDescriptionUpdated(const ProgramId & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void Fetcher::errorFetchingGroup(const GroupId & _t1, const Error & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void Fetcher::errorFetchingChannel(const ChannelId & _t1, const Error & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void Fetcher::imageDownloadFinished(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Fetcher::errorDownloadingImage(const QString & _t1, const Error & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}
QT_WARNING_POP
