/****************************************************************************
** Meta object code from reading C++ file 'TellySkoutSettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../TellySkoutSettings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TellySkoutSettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18TellySkoutSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto TellySkoutSettings::qt_create_metaobjectdata<qt_meta_tag_ZN18TellySkoutSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TellySkoutSettings",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "deleteProgramAfterChanged",
        "",
        "fetcherChanged",
        "programHeightChanged",
        "columnWidthChanged",
        "fontSizeChanged",
        "showDateSelectionChanged",
        "tvSpielfilmPrefetchChanged",
        "xmltvFileChanged",
        "deleteProgramAfter",
        "isDeleteProgramAfterImmutable",
        "defaultDeleteProgramAfterValue",
        "fetcher",
        "isFetcherImmutable",
        "programHeight",
        "isProgramHeightImmutable",
        "defaultProgramHeightValue",
        "columnWidth",
        "isColumnWidthImmutable",
        "defaultColumnWidthValue",
        "fontSize",
        "isFontSizeImmutable",
        "defaultFontSizeValue",
        "showDateSelection",
        "isShowDateSelectionImmutable",
        "defaultShowDateSelectionValue",
        "tvSpielfilmPrefetch",
        "isTvSpielfilmPrefetchImmutable",
        "defaultTvSpielfilmPrefetchValue",
        "xmltvFile",
        "isXmltvFileImmutable"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'deleteProgramAfterChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fetcherChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'programHeightChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'columnWidthChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontSizeChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showDateSelectionChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tvSpielfilmPrefetchChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'xmltvFileChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'deleteProgramAfter'
        QtMocHelpers::PropertyData<uint>(14, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'isDeleteProgramAfterImmutable'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultDeleteProgramAfterValue'
        QtMocHelpers::PropertyData<uint>(16, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fetcher'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'isFetcherImmutable'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'programHeight'
        QtMocHelpers::PropertyData<uint>(19, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'isProgramHeightImmutable'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultProgramHeightValue'
        QtMocHelpers::PropertyData<uint>(21, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'columnWidth'
        QtMocHelpers::PropertyData<uint>(22, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'isColumnWidthImmutable'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultColumnWidthValue'
        QtMocHelpers::PropertyData<uint>(24, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fontSize'
        QtMocHelpers::PropertyData<uint>(25, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isFontSizeImmutable'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultFontSizeValue'
        QtMocHelpers::PropertyData<uint>(27, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'showDateSelection'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'isShowDateSelectionImmutable'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultShowDateSelectionValue'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'tvSpielfilmPrefetch'
        QtMocHelpers::PropertyData<uint>(31, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'isTvSpielfilmPrefetchImmutable'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultTvSpielfilmPrefetchValue'
        QtMocHelpers::PropertyData<uint>(33, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'xmltvFile'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'isXmltvFileImmutable'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<TellySkoutSettings, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject TellySkoutSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<KConfigSkeleton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TellySkoutSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TellySkoutSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18TellySkoutSettingsE_t>.metaTypes,
    nullptr
} };

void TellySkoutSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TellySkoutSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->deleteProgramAfterChanged(); break;
        case 1: _t->fetcherChanged(); break;
        case 2: _t->programHeightChanged(); break;
        case 3: _t->columnWidthChanged(); break;
        case 4: _t->fontSizeChanged(); break;
        case 5: _t->showDateSelectionChanged(); break;
        case 6: _t->tvSpielfilmPrefetchChanged(); break;
        case 7: _t->xmltvFileChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TellySkoutSettings::*)()>(_a, &TellySkoutSettings::deleteProgramAfterChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TellySkoutSettings::*)()>(_a, &TellySkoutSettings::fetcherChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TellySkoutSettings::*)()>(_a, &TellySkoutSettings::programHeightChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TellySkoutSettings::*)()>(_a, &TellySkoutSettings::columnWidthChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TellySkoutSettings::*)()>(_a, &TellySkoutSettings::fontSizeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TellySkoutSettings::*)()>(_a, &TellySkoutSettings::showDateSelectionChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TellySkoutSettings::*)()>(_a, &TellySkoutSettings::tvSpielfilmPrefetchChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TellySkoutSettings::*)()>(_a, &TellySkoutSettings::xmltvFileChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->deleteProgramAfter(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isDeleteProgramAfterImmutable(); break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->defaultDeleteProgramAfterValue(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->fetcher(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isFetcherImmutable(); break;
        case 5: *reinterpret_cast<uint*>(_v) = _t->programHeight(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isProgramHeightImmutable(); break;
        case 7: *reinterpret_cast<uint*>(_v) = _t->defaultProgramHeightValue(); break;
        case 8: *reinterpret_cast<uint*>(_v) = _t->columnWidth(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isColumnWidthImmutable(); break;
        case 10: *reinterpret_cast<uint*>(_v) = _t->defaultColumnWidthValue(); break;
        case 11: *reinterpret_cast<uint*>(_v) = _t->fontSize(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->isFontSizeImmutable(); break;
        case 13: *reinterpret_cast<uint*>(_v) = _t->defaultFontSizeValue(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->showDateSelection(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->isShowDateSelectionImmutable(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->defaultShowDateSelectionValue(); break;
        case 17: *reinterpret_cast<uint*>(_v) = _t->tvSpielfilmPrefetch(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isTvSpielfilmPrefetchImmutable(); break;
        case 19: *reinterpret_cast<uint*>(_v) = _t->defaultTvSpielfilmPrefetchValue(); break;
        case 20: *reinterpret_cast<QString*>(_v) = _t->xmltvFile(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->isXmltvFileImmutable(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDeleteProgramAfter(*reinterpret_cast<uint*>(_v)); break;
        case 3: _t->setFetcher(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setProgramHeight(*reinterpret_cast<uint*>(_v)); break;
        case 8: _t->setColumnWidth(*reinterpret_cast<uint*>(_v)); break;
        case 11: _t->setFontSize(*reinterpret_cast<uint*>(_v)); break;
        case 14: _t->setShowDateSelection(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setTvSpielfilmPrefetch(*reinterpret_cast<uint*>(_v)); break;
        case 20: _t->setXmltvFile(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *TellySkoutSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TellySkoutSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TellySkoutSettingsE_t>.strings))
        return static_cast<void*>(this);
    return KConfigSkeleton::qt_metacast(_clname);
}

int TellySkoutSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KConfigSkeleton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void TellySkoutSettings::deleteProgramAfterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TellySkoutSettings::fetcherChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TellySkoutSettings::programHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TellySkoutSettings::columnWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TellySkoutSettings::fontSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TellySkoutSettings::showDateSelectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void TellySkoutSettings::tvSpielfilmPrefetchChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void TellySkoutSettings::xmltvFileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
