/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<TellySkoutSettings.h>)
#  include <TellySkoutSettings.h>
#endif
#if __has_include(<channelsmodel.h>)
#  include <channelsmodel.h>
#endif
#if __has_include(<channelsproxymodel.h>)
#  include <channelsproxymodel.h>
#endif
#if __has_include(<fetcher.h>)
#  include <fetcher.h>
#endif
#if __has_include(<groupsmodel.h>)
#  include <groupsmodel.h>
#endif
#if __has_include(<programsmodel.h>)
#  include <programsmodel.h>
#endif
#if __has_include(<programsproxymodel.h>)
#  include <programsproxymodel.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_TellySkout()
{
    qmlRegisterModule("org.kde.TellySkout", 254, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<ChannelsModel>("org.kde.TellySkout", 254);
    qmlRegisterTypesAndRevisions<ChannelsProxyModel>("org.kde.TellySkout", 254);
    qmlRegisterTypesAndRevisions<Fetcher>("org.kde.TellySkout", 254);
    qmlRegisterTypesAndRevisions<GroupsModel>("org.kde.TellySkout", 254);
    QMetaType::fromType<KConfigSkeleton *>().id();
    QMetaType::fromType<KCoreConfigSkeleton *>().id();
    qmlRegisterEnum<KCoreConfigSkeleton::DisambiguateConstructor>("KCoreConfigSkeleton::DisambiguateConstructor");
    qmlRegisterTypesAndRevisions<ProgramsModel>("org.kde.TellySkout", 254);
    qmlRegisterTypesAndRevisions<ProgramsProxyModel>("org.kde.TellySkout", 254);
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    QMetaType::fromType<QAbstractProxyModel *>().id();
    QMetaType::fromType<QSortFilterProxyModel *>().id();
    qmlRegisterTypesAndRevisions<TellySkoutSettings>("org.kde.TellySkout", 254);
    QT_WARNING_POP
    qmlRegisterModule("org.kde.TellySkout", 254, 254);
}

static const QQmlModuleRegistration orgkdeTellySkoutRegistration("org.kde.TellySkout", qml_register_types_org_kde_TellySkout);
